/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2015 The International Cooperation for the Integration of
 * Processes in  Prepress, Press and Postpress (CIP4).  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        The International Cooperation for the Integration of
 *        Processes in  Prepress, Press and Postpress (www.cip4.org)"
 *    Alternately, this acknowledgment mrSubRefay appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of
 *    Processes in  Prepress, Press and Postpress" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4",
 *    nor may "CIP4" appear in their name, without prior writtenrestartProcesses()
 *    permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For
 * details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR
 * THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the The International Cooperation for the Integration
 * of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses()
 * copyright (c) 1999-2001, Heidelberger Druckmaschinen AG
 * copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the
 * Integration of Processes in  Prepress, Press and Postpress , please see
 * <http://www.cip4.org/>.
 *
 */
package org.cip4.jdflib.extensions.xjdfwalker;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.cip4.jdflib.util.ContainerUtil;
import org.cip4.jdflib.util.MyLong;
import org.cip4.jdflib.util.MyPair;
import org.cip4.jdflib.util.thread.RegularJanitor;
import org.cip4.jdflib.util.thread.TimeSweeper;

/**
 * class to maintain a map of refID to type values so that we can correctly modify @Type in responses
 * messages older than 4242 seconds are discarded
 * 
 * @author rainer prosi
 *
 */
public class XJMFTypeMap implements Runnable
{
	private class TimePair extends MyPair<String, MyLong>
	{
		/**
		 * 
		 * @param val
		 */
		TimePair(String val)
		{
			super(val, new MyLong(System.currentTimeMillis()));
		}

		/**
		 * age in seconds
		 * @return
		 */
		long age()
		{
			return (System.currentTimeMillis() - b.i) / 1000;
		}

		/**
		 * age in seconds
		 * @return
		 */
		void touch()
		{
			b.i = System.currentTimeMillis();
		}

	}

	private XJMFTypeMap()
	{
		super();
		map = new HashMap<String, TimePair>();
		RegularJanitor.getJanitor().addSweeper(new TimeSweeper(4242, this), true);
		RegularJanitor.getJanitor().startSweep(42);
	}

	private static XJMFTypeMap theMap = null;
	private final Map<String, TimePair> map;

	/**
	 * 
	 * @return
	 */
	public static XJMFTypeMap getMap()
	{
		if (theMap == null)
			theMap = new XJMFTypeMap();

		return theMap;
	}

	/**
	 * 
	 * @param key
	 * @param value
	 * @return
	 */
	public String put(String key, String value)
	{
		TimePair put = map.put(key, new TimePair(value));
		return put == null ? null : put.a;
	}

	/**
	 * 
	 * @param key
	 * @return
	 */
	public String remove(Object key)
	{
		TimePair put = map.remove(key);
		return put == null ? null : put.a;
	}

	/**
	 * 
	 * @param key
	 * @return
	 */
	public String get(Object key)
	{
		TimePair put = map.get(key);
		if (put != null)
		{
			put.touch();
			return put.a;
		}
		return null;
	}

	/**
	 * 
	 */
	public void clear()
	{
		map.clear();
	}

	/**
	 * called by the sweeper
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		Vector<String> keys = ContainerUtil.getKeyVector(map);
		if (keys == null)
			return;
		for (String key : keys)
		{
			TimePair val = map.get(key);
			if (val != null && val.age() > 4242l)
			{
				map.remove(key);
			}
		}
	}

	/**
	 * 
	 */
	public static void shutDown()
	{
		if (theMap != null)
			theMap.clear();
		theMap = null;
		RegularJanitor.feierabend();
	}

	/**
	 * 
	 * @return
	 */
	public int size()
	{
		return map.size();
	}
}
